package jesse.keeblarcraft.gui.ShopKeeperGUI;

import jesse.keeblarcraft.GuiMgr.ShopKeeperHandler;
import jesse.keeblarcraft.Keeblarcraft;
import jesse.keeblarcraft.ClientHelpers.Helper;
import jesse.keeblarcraft.Utils.CustomSlot;
import jesse.keeblarcraft.gui.Generics.ClickableLayer;
import net.minecraft.client.gui.DrawContext;
import net.minecraft.client.gui.screen.ingame.HandledScreen;
import net.minecraft.entity.player.PlayerInventory;
import net.minecraft.text.Text;
import net.minecraft.util.Identifier;

import java.util.ArrayList;
import java.util.List;

public class ShopKeeperMenu extends HandledScreen<ShopKeeperHandler> {
    private static final Identifier MENU_TEXTURE = new Identifier(Keeblarcraft.MOD_ID, "textures/gui/shopkeeper_gui.jpeg");
    private static final Identifier ADD_BUTTON = new Identifier(Keeblarcraft.MOD_ID, "textures/gui/shopkeeper_add_icon.jpeg");
    private static final Identifier NEW_ITEM_MENU = new Identifier(Keeblarcraft.MOD_ID, "textures/gui/shopkeeper_item_slot.jpeg");
    private int x = 0;
    private int y = 0;
    private final List<ClickableLayer> layers = new ArrayList<>();

    public ShopKeeperMenu(ShopKeeperHandler handler, PlayerInventory inventory, Text title) {
        super(handler, inventory, title);
    }

    @Override
    protected void init() {
        super.init();
        titleY = 1000; //begone from screen
        playerInventoryTitleY = 1000; //begone from screen

        // Set the main shop keeping menu variables here
        this.backgroundWidth = 256;
        this.backgroundHeight = 256;
        this.x = (width - backgroundWidth) / 2;
        this.y = (height - backgroundHeight) / 2;

        // Add all layers below
        AddNewButton(ADD_BUTTON, Text.of("ADD ITEM BUTTON"), x, y, 64, 64);
    }

    @Override
    protected void drawBackground(DrawContext context, float delta, int mouseX, int mouseY) {
        // Draw the menu texture first
//        context.drawTexture(MENU_TEXTURE, this.x, this.y, 0, 0, this.backgroundWidth, this.backgroundHeight);
        context.drawTexture(MENU_TEXTURE, this.x, this.y, 0, 0, this.backgroundWidth, this.backgroundHeight, this.backgroundWidth, this.backgroundHeight);
//        context.drawTexture(NEW_ITEM_MENU, this.x, this.y, 0, 0, 32, 24);
//        context.drawTexture(NEW_ITEM_MENU, this.x, this.y, 0, 0, 32, 24, 32, 24);

//        this.getScreenHandler().ShowSlot(Text.of("test"));

        for (ClickableLayer layer : layers) {
            layer.renderButton(context, mouseX, mouseY, delta);

            for (ClickableLayer subLayer : layer.GetSubLayers()) {
                subLayer.renderButton(context, mouseX, mouseY, delta);
            }

            // We only notify the parent to update sub layers. It must manage its own sub layers
            layer.UpdateSubLayers();
        }
    }

    // Takes an image and adds it to the background draw list and saves coordinates to return to a callback if it is clicked
    private void AddNewButton(Identifier iconId, Text layerName, int drawX, int drawY, int imgWidth, int imgHeight) {
        layers.add(new NewItemButton(iconId, layerName, imgWidth, imgHeight, drawX, drawY));
    }

    @Override
     public void drawForeground(DrawContext context, int mouseX, int mouseY) {
         // We override this function to intentionally do nothing
         // If in the future we want, we would draw the foreground and TITLES with this!
     }

    @Override
    public boolean mouseClicked(double mouseX, double mouseY, int button) {
        for (ClickableLayer layer : layers) {
            // Check main layer first
            if (Helper.WithinBounds(layer.getX(), layer.getY(), layer.getWidth(), layer.getHeight(), mouseX, mouseY)) {
                layer.mouseClicked(mouseX, mouseY, button);
            }

            for (ClickableLayer subLayer : layer.GetSubLayers()) {

                if (subLayer.visible && Helper.WithinBounds(subLayer.getX(), subLayer.getY(), subLayer.getWidth(), subLayer.getHeight(), mouseX, mouseY)) {
                    subLayer.mouseClicked(mouseX, mouseY, button);
                }
            }
        }

        return true;
    }

    // Check if a click point is within the context of a square (square = x, y, w, h)
//    public static boolean WithinBounds(double x, double y, double w, double h, double clickX, double clickY) {
////        System.out.println("Mouse click X Y: " + clickX + " " + clickY);
////        System.out.println("Rectangle X Y W H: " + x + " " + y + " " + w + " " + h);
//        return (clickX >= x) && (clickX <= x + w) && (clickY >= y) && (clickY <= y + h);
//    }

    @Override
    public void render(DrawContext context, int mouseX, int mouseY, float delta) {
        renderBackground(context, mouseX, mouseY, delta);
        super.render(context, mouseX, mouseY, delta);
        drawMouseoverTooltip(context, mouseX, mouseY);
    }
}
